local addonName = "VotansFishFillet"
local useCallProtectedFunction = IsProtectedFunction("UseItem")

local function IsItemFish(bagId, slotIndex)
  if bagId ~= BAG_BACKPACK then return false end

  local itemType = GetItemType(bagId, slotIndex)
  if ITEMTYPE_FISH == itemType then
    local usable, onlyFromActionSlot = IsItemUsable(bagId, slotIndex)
    local canInteractWithItem = CanInteractWithItem(bagId, slotIndex)
    return usable and not onlyFromActionSlot and canInteractWithItem
  end
  return false
end

local function UseInventorySlot(inventorySlot, slotActions)
  local FiletFish
  local count = 0
  local itemLink
  local slotIndex
  local orgAddMessage = CENTER_SCREEN_ANNOUNCE.AddMessage

  local function NoMessage(self, category, ...)
    if category ~= EVENT_DISPLAY_ANNOUNCEMENT then orgAddMessage(self, category, ...) end
  end
  local function AbortAction(...)
    CENTER_SCREEN_ANNOUNCE.AddMessage = orgAddMessage
    EVENT_MANAGER:UnregisterForEvent(addonName, EVENT_INVENTORY_FULL_UPDATE)
    EVENT_MANAGER:UnregisterForEvent(addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE)
    ShowMouse()
  end

  local function InventoryStateChange(oldState, newState)
    if newState == SCENE_HIDING then
      count = 0
      AbortAction()
      INVENTORY_FRAGMENT:UnregisterCallback("StateChange", InventoryStateChange)
    end
  end

  local function EndAction(...)
    AbortAction()
    INVENTORY_FRAGMENT:UnregisterCallback("StateChange", InventoryStateChange)
  end

  local function SlotUpdate(eventCode, bagId, _, isNew, ...)
    if bagId ~= BAG_BACKPACK then return end
    if isNew then
      count = count - 1
      if count > 0 then
        -- Not zo_callLater: if there is a delay already, it will not registered twice
        EVENT_MANAGER:RegisterForUpdate(addonName, 40, FiletFish)
      else
        EndAction()
      end
    end
  end

  -- After reorg of the inventory, e.g. after new entry "fish", find the stack again
  local function SlotFullUpdate(eventCode, bagId, ...)
    if bagId ~= BAG_BACKPACK then return end
    local bagSlots = GetBagSize(BAG_BACKPACK) - 1
    local found = false
    local git = GetItemLink
    local gii = GetItemInfo
    for index = 0, bagSlots do
      if itemLink == git(BAG_BACKPACK, index) then
        found = true
        slotIndex = index
        local _, stack = gii(BAG_BACKPACK, slotIndex)
        if stack == (count-1) then
          break
        end
      end
    end
    if found then
      SlotUpdate(eventCode, BAG_BACKPACK, slotIndex, true)
    else
      EndAction()
    end
  end

  FiletFish = function()
    EVENT_MANAGER:UnregisterForUpdate(addonName)
    CENTER_SCREEN_ANNOUNCE.AddMessage = orgAddMessage
    if not CheckInventorySpaceSilently(1) then
      EndAction()
      ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, SI_INVENTORY_ERROR_INVENTORY_FULL)
      return false
    end
    if IsItemFish(BAG_BACKPACK, slotIndex) then
      EVENT_MANAGER:RegisterForEvent(addonName, EVENT_INVENTORY_FULL_UPDATE, SlotFullUpdate)
      EVENT_MANAGER:RegisterForEvent(addonName, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, SlotUpdate)
      INVENTORY_FRAGMENT:RegisterCallback("StateChange", InventoryStateChange)
      CENTER_SCREEN_ANNOUNCE.AddMessage = NoMessage

      if useCallProtectedFunction then
        if not CallSecureProtected("UseItem", BAG_BACKPACK, slotIndex) then
          EndAction()
          PlaySound(SOUNDS.NEGATIVE_CLICK)
        end
      else
        UseItem(BAG_BACKPACK, slotIndex)
      end
      -- inventorySlot is unvalid afterwards, because the position could have changed due to the new entry "fish"
    else
      EndAction()
    end
    return false
  end

  local bagId, _
  bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)

  if IsItemFish(bagId, slotIndex) then
    _, count = GetItemInfo(bagId, slotIndex)
    if count > 0 then
      itemLink = GetItemLink(bagId, slotIndex)
      EVENT_MANAGER:RegisterForUpdate(addonName, 40, FiletFish)
      slotActions:Clear()
      ClearCursor()
      ClearMenu()
      HideMouse()
      return true
    else
      itemLink = ""
      EndAction()
    end
  end
  return false
end

do
  local function AddFiletAll(inventorySlot, slotActions)
    local bagId, slotIndex = ZO_Inventory_GetBagAndIndex(inventorySlot)
    if IsItemFish(bagId, slotIndex) and CheckInventorySpaceSilently(1) then
      slotActions:AddSlotAction(SI_VOTANS_FILET_FISH_ALL, function(...) return UseInventorySlot(inventorySlot, slotActions) end, "primary", nil, {visibleWhenDead = false})
    end
    return false
  end

  ZO_PreHook("ZO_InventorySlot_DiscoverSlotActionsFromActionList", AddFiletAll)
end
